@@ -12,11 +12,10 @@ import subprocess
 import webbrowser
 import psutil
 
-import esptool
 import serial
 
 from PySide6 import QtCore
-from PySide6.QtGui import QPixmap, QCursor
+from PySide6.QtGui import QPixmap, QCursor, QIcon
 from PySide6.QtWidgets import (QPushButton, QApplication,
                                QVBoxLayout, QHBoxLayout, QDialog, QLabel,
                                QMessageBox, QComboBox, QProgressBar)
@@ -26,9 +25,14 @@ import meshtastic.serial_interface
 
 from meshtastic_flasher.version import __version__
 from meshtastic_flasher.advanced_form import AdvancedForm
+from meshtastic_flasher.esptool_form import EsptoolForm
+from meshtastic_flasher.settings import Settings
 import meshtastic_flasher.util
 
 MESHTASTIC_LOGO_FILENAME = "logo.png"
+COG_FILENAME = "cog.svg"
+HELP_FILENAME = "help.svg"
+BUTTON_ICON_SIZE = QtCore.QSize(24, 24)
 
 # windows does not like this one
 if platform.system() == "Linux":
@@ -51,10 +55,15 @@ class Form(QDialog):
         self.firmware_version = None
         self.nrf = False
         self.device = None
-        self.update_only = False
+        self.update_only = None
+        self.device_file = None
+        self.system_info_file = None
+        self.bin_file = None
         self.detected_meshtastic_version = None
 
         self.advanced_form = AdvancedForm()
+        self.esptool_form = EsptoolForm()
+        self.settings = Settings()
 
         update_available = ''
         if meshtastic_flasher.util.check_if_newer_version():
@@ -75,7 +84,12 @@ class Form(QDialog):
         # Note: The text of the buttons is done in the styles, need to override it
         self.select_detect.setStyleSheet("text-transform: none")
 
-        self.help_button = QPushButton("?")
+        self.help_button = QPushButton()
+        help_icon = QIcon(meshtastic_flasher.util.get_path(HELP_FILENAME))
+        self.help_button.setIcon(help_icon)
+        self.help_button.setIconSize(BUTTON_ICON_SIZE)
+        self.help_button.setFixedWidth(42)
+        self.help_button.setCursor(QCursor(QtCore.Qt.PointingHandCursor))
         self.help_button.setToolTip("Click for help.")
 
         self.select_port = QComboBox()
@@ -125,6 +139,13 @@ class Form(QDialog):
         self.label_detected_meshtastic_version = QLabel(self)
         self.label_detected_meshtastic_version.setText("")
 
+        self.settings_cog = QPushButton()
+        cog_icon = QIcon(meshtastic_flasher.util.get_path(COG_FILENAME))
+        self.settings_cog.setIcon(cog_icon)
+        self.settings_cog.setIconSize(BUTTON_ICON_SIZE)
+        self.settings_cog.setFixedWidth(42)
+        self.settings_cog.setCursor(QCursor(QtCore.Qt.PointingHandCursor))
+
         # Create layout and add widgets
         main_layout = QVBoxLayout()
         main_layout.setContentsMargins(0, 0, 0, 0)
@@ -138,6 +159,7 @@ class Form(QDialog):
         detect_layout.addWidget(self.get_versions_button)
         detect_layout.addWidget(self.select_detect)
         detect_layout.addWidget(self.help_button)
+        detect_layout.addWidget(self.settings_cog)
         detect_layout.setContentsMargins(0, 0, 0, 0)
         detect_layout.addStretch(1)
 
@@ -175,9 +197,12 @@ class Form(QDialog):
         self.setLayout(main_layout)
 
         # move version
-        self.label_detected_meshtastic_version.move(45, 270)
+        self.label_detected_meshtastic_version.move(30, 270)
         self.label_detected_meshtastic_version.show()
 
+        #self.settings_cog.move(400, 270)
+        #self.settings_cog.show()
+
         # Add button signals to slots
         self.logo.mousePressEvent = self.logo_clicked
         self.get_versions_button.clicked.connect(self.get_versions)
@@ -187,6 +212,7 @@ class Form(QDialog):
         self.select_detect.clicked.connect(self.detect)
         self.select_flash.clicked.connect(self.flash_stuff)
         self.select_firmware_version.currentTextChanged.connect(self.on_select_firmware_changed)
+        self.settings_cog.mousePressEvent = self.run_settings
 
         # pre-populate the versions that have already been downloaded and unzipped
         self.get_versions_from_disk()
@@ -196,7 +222,7 @@ class Form(QDialog):
         """Deal with a key press"""
         if event.key() == QtCore.Qt.Key_A:
             print("A was pressed... showing advanced options form")
-            self.show_advanced_options()
+            self.advanced_form.show()
         elif event.key() == QtCore.Qt.Key_D:
             print("D was pressed...")
             self.detect()
@@ -210,8 +236,22 @@ class Form(QDialog):
             print("Q was pressed... so quitting")
             QApplication.quit()
         elif event.key() == QtCore.Qt.Key_T:
-            print("T was pressed... so quitting")
+            print("T was pressed... so showing tips")
             self.tips()
+        elif event.key() == QtCore.Qt.Key_S:
+            print("S was pressed... showing settings form")
+            self.run_settings(None)
+
+
+    # pylint: disable=unused-argument
+    def run_settings(self, event):
+        """Run the settings form"""
+        if self.select_port.currentText() == '':
+            print('We do not have a port.')
+            QMessageBox.information(self, "Info", "Need a port before running Settings. Click DETECT DEVICE.")
+        else:
+            self.port = self.select_port.currentText()
+            self.settings.run(port=self.port)
 
 
     def on_select_firmware_changed(self, value):
@@ -313,13 +353,19 @@ class Form(QDialog):
     def tips(self):
         """Show tips"""
         print("tips")
-        QMessageBox.information(self, "Info", ("Tips:\n\n"
-                                "If having issues flashing the device, be sure there is only one device connected\n"
-                                "and no other applications are using that communications port.\n\n"
-                                "If still having problems, unplug the device, then re-plugin the device.\n\n"
-                                "If still having problems, try rebooting the pc.\n\n"
-                                "If you get a 'Critical Fault #6' on a T-Beam, it probably means you need to use\n"
-                                "v1.1 or v2.1.1.6 firmware.\n\n"))
+        msg_box = QMessageBox()
+        msg_box.setTextFormat(QtCore.Qt.RichText)  # this is what makes the links clickable
+        msg_box.setText("Tips:<br><br>"
+                        "If having issues flashing the device, be sure there is only one device connected "
+                        "and no other applications are using that communications port.<br><br>"
+                        "If still having problems, unplug the device, then re-plugin the device.<br><br>"
+                        "If still having problems, may need to install driver. "
+                        "See <a href='https://meshtastic.org/docs/getting-started/flashing-esp32'>Flashing</a> for more info.<br><br>"
+                        "If still having problems, try rebooting the pc.<br><br>"
+                        "If you get a 'Critical Fault #6' on a T-Beam, it probably means you need to use "
+                        "v1.1 or v2.1.1.6 firmware.")
+        msg_box.setStandardButtons(QMessageBox.Ok)
+        msg_box.exec()
 
 
     # pylint: disable=unused-argument
@@ -336,6 +382,17 @@ class Form(QDialog):
         webbrowser.open('https://meshtastic.org/docs/hardware')
 
 
+    def do_esptool(self):
+        """do_esptool"""
+        print(f"do esptool update_only:{self.update_only}")
+        # if they do another session, clear contents of prior session
+        self.esptool_form.text.clear()
+        self.esptool_form.show()
+
+        if self.update_only is not None:
+            self.esptool_form.start(update_only=self.update_only, port=self.port, device_file=self.device_file, system_info_file=self.system_info_file, bin_file=self.bin_file)
+
+
     def hotkeys(self):
         """Show hotkeys"""
         print("hotkeys")
@@ -344,16 +401,11 @@ class Form(QDialog):
                                 "D - Detect\n"
                                 "G - Get versions\n"
                                 "H - Hotkeys\n"
+                                "S - Settings\n"
                                 "T - Tips\n"
                                 "Q - Quit\n"))
 
 
-    def show_advanced_options(self):
-        """Advanced Options"""
-        print("advanced options")
-        self.advanced_form.show()
-
-
     def detect_devices(self):
         """Detect devices"""
         supported_devices_detected = meshtastic_flasher.util.wrapped_detect_supported_devices()
@@ -727,7 +779,10 @@ class Form(QDialog):
         self.port = None
         self.nrf = False
         self.device = None
-        self.update_only = False
+        self.device_file = None
+        self.system_info_file = None
+        self.bin_file = None
+        self.update_only = None
         self.select_device.clear()
         self.select_port.clear()
         self.select_flash.setEnabled(False)
@@ -844,70 +899,6 @@ class Form(QDialog):
         webbrowser.open('https://meshtastic.org')
 
 
-    def flash_esp32_update_only_step1(self, percent_complete):
-        """Do step 1 of 2 for esp32 update_only"""
-        print("Step 1/2 esp32 update_only")
-        device_file = f"{self.firmware_version}/firmware-{self.device}-{self.firmware_version}.bin"
-        command = ["--port", self.port, "write_flash", "0x10000", device_file]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
-    def flash_esp32_update_only_step2(self, percent_complete):
-        """Do step 2 of 2 for esp32 update_only"""
-        print("Step 2/2 esp32 update_only")
-        command = ["--port", self.port, "erase_region", "0xe000", "0x2000"]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
-    def flash_esp32_full_step1(self, percent_complete):
-        """Do step 1 of 4 for esp32 full flash"""
-        print("Step 1/4 esp32 full")
-        command = ["--port", self.port, "erase_flash"]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
-    def flash_esp32_full_step2(self, percent_complete):
-        """Do step 2 of 4 for esp32 full flash"""
-        print("Step 2/4 esp32 full")
-        system_info_file = f"{self.firmware_version}/system-info.bin"
-        command = ["--port", self.port, "write_flash", "0x1000", system_info_file]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
-    def flash_esp32_full_step3(self, percent_complete):
-        """Do step 3 of 4 for esp32 full flash"""
-        print("Step 3/4 esp32 full")
-        bin_file = f"{self.firmware_version}/spiffs-{self.firmware_version}.bin"
-        command = ["--port", self.port, "write_flash", "0x00390000", bin_file]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
-    def flash_esp32_full_step4(self, percent_complete):
-        """Do step 4 of 4 for esp32 full flash"""
-        print("Step 4/4 esp32 full")
-        device_file = f"{self.firmware_version}/firmware-{self.device}-{self.firmware_version}.bin"
-        command = ["--port", self.port, "write_flash", "0x10000", device_file]
-        print(f"ESPTOOL Using command:{' '.join(command)}")
-        esptool.main(command)
-        self.progress.setValue(percent_complete)
-        QApplication.processEvents()
-
-
     def flash_nrf52(self):
         """Flash nrf52 devices"""
         print("Flash nrf52")
@@ -939,18 +930,21 @@ class Form(QDialog):
         """
         want_to_proceed = False
         verb = 'flash'
-        all_settings_msg = 'NOTE: All Meshtastic settings will be erased.'
+        all_settings_msg = ''
+        if update_only_message == '':
+            all_settings_msg = 'NOTE: All Meshtastic settings will be erased.'
         if self.nrf:
             verb = 'copy'
             update_only_message = ''
             all_settings_msg = ''
         confirm_msg = f'Are you sure you want to {update_only_message}{verb}\n{self.firmware_version}\n'
-        confirm_msg += f'{self.port}\n{self.device}?\n{all_settings_msg}'
-        reply = QMessageBox.question(self, 'Flash', confirm_msg,
-                                     QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+        confirm_msg += f'{self.port}\n{self.device}?\n\n{all_settings_msg}'
+        reply = QMessageBox.question(self, 'Flash', confirm_msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
         if reply == QMessageBox.Yes:
             want_to_proceed = True
             print("User confirmed they want to flash")
+        else:
+            print("User does not want to flash")
         return want_to_proceed
 
 
@@ -960,13 +954,12 @@ class Form(QDialog):
         """
         want_to_check = False
         msg = 'Does the device currently have Meshtastic version 1.2 or greater?'
-        reply = QMessageBox.question(self, 'Question', msg,
-                                     QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+        reply = QMessageBox.question(self, 'Question', msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
         if reply == QMessageBox.Yes:
             want_to_check = True
-            print("User confirmed they want to check using the Meshtastic python method")
+            print("User confirmed the device has Meshtastic so we will check using the Meshtastic python method")
         else:
-            print("User declined detection using the Meshtastic python method")
+            print("User said no to device having Meshtastic on the device so no detection using the Meshtastic python method")
         return want_to_check
 
 
@@ -996,19 +989,10 @@ class Form(QDialog):
                 QMessageBox.information(self, "Info", "File was copied.\nWait for the device to reboot.")
             else:
                 # esp32 devices
+                self.device_file = f"{self.firmware_version}/firmware-{self.device}-{self.firmware_version}.bin"
                 if self.update_only:
-                    self.flash_esp32_update_only_step1(50)
-                    self.flash_esp32_update_only_step2(100)
-                    print("esp32 update_only complete")
+                    self.do_esptool()
                 else:
-                    # move the progress bar a bit so user sees something is happening
-                    self.progress.setValue(10)
-                    QApplication.processEvents()
-
-                    self.flash_esp32_full_step1(25)
-                    self.flash_esp32_full_step2(50)
-                    self.flash_esp32_full_step3(75)
-                    self.flash_esp32_full_step4(100)
-                    print("esp32 full complete")
-
-                QMessageBox.information(self, "Info", "Flashed")
+                    self.system_info_file = f"{self.firmware_version}/system-info.bin"
+                    self.bin_file = f"{self.firmware_version}/spiffs-{self.firmware_version}.bin"
+                    self.do_esptool()
