@@ -173,7 +173,7 @@ def is_windows11():
     """Detect if Windows 11"""
     is_win11 = False
     if platform.system() == "Windows":
-        if int(platform.release()) >= 10:
+        if float(platform.release()) >= 10.0:
             patch = platform.version().split('.')[2]
             print(f'patch:{patch}')
             # in case they add some number suffix later, just get first 5 chars of patch
@@ -185,3 +185,16 @@ def is_windows11():
                 print(f'problem detecting win11 e:{e}')
     print(f'is_win11:{is_win11}')
     return is_win11
+
+
+def zero_if_blank(some_input):
+    """If some_input is blank, then return '0' else return a string after converting to an integer."""
+    retval = 0
+    if some_input is None or some_input == '':
+        retval = 0
+    else:
+        try:
+            retval = int(some_input)
+        except:
+            pass
+    return f'{retval}'
